#!/usr/bin/python2.7

import os
import serial
from struct import *

from tabulate import tabulate

import math

import numpy as np

import time

ADC_COUNTS=4096


ser = serial.Serial('/dev/ttyACM0') 
ser.flushInput()
ser.flushOutput()

packet_size=0
packet_start=0

packet_sum=""

offsetV=2080
offsetI=2062

V1cal=0.236
I1cal=0.0068

# 1 Min data buffers
V1_data=[]
V2_data=[]
V3_data=[]
I1_data=[]
I2_data=[]
I3_data=[]

array_start_time=0
packet_count=0

crossing_bool = 0

def Calc_VI():

	sumI=0
	sumV=0
	sumP=0
	
	global crossing_bool
	global packet_count
	global array_start_time
	
	crossing_bool=0
	
	crossCount = 0                            #Used to measure number of times threshold is crossed.
	numberOfSamples = 0                        #This is now incremented  

	#-------------------------------------------------------------------------------------------------------------------------
	# 1) Waits for the waveform to be close to 'zero' (mid-scale adc) part in sin curve.
	#-------------------------------------------------------------------------------------------------------------------------
	st=0;                                  #an indicator to exit the while loop
	
	exit_counter=0
	startV=0
	
	while(st==0):                                 #the while loop...

		startV = V1_data[-1024+exit_counter]
		if ((startV < (ADC_COUNTS*0.6)) and (startV > (ADC_COUNTS*0.5)) and (startV>V1_data[-1024+exit_counter-1])): #Check for rising V and centered
			st=1;  #check its within range
			crossing_bool=1 #set flag, that it is inside crossing
		if (exit_counter>1020):
			st = 1;
		exit_counter=exit_counter+1

	start_point=exit_counter-1
	
	#-------------------------------------------------------------------------------------------------------------------------
	# 2) Main measurement loop
	#------------------------------------------------------------------------------------------------------------------------- 

	sample_counter=0
	exit=0
	
	while (exit==0 and (sample_counter<512)) :

		global offsetV
		global offsetI
		
		sample_counter=sample_counter+1                       #Count number of times looped.
		#lastFilteredV = filteredV;               #Used for delay/phase compensation

		#-----------------------------------------------------------------------------
		# A) Read in raw voltage and current samples
		#-----------------------------------------------------------------------------
		sampleV = V1_data[-1024+start_point+sample_counter]                #Read in raw voltage signal
		sampleI = I1_data[-1024+start_point+sample_counter]                 #Read in raw current signal

		#-----------------------------------------------------------------------------
		# B) Apply digital low pass filters to extract the 2.5 V or 1.65 V dc offset,
		#     then subtract this - signal is now centred on 0 counts.
		#-----------------------------------------------------------------------------
		#offsetV = offsetV + ((sampleV-offsetV)/4096)
		filteredV = sampleV - offsetV
		#offsetI = offsetI + ((sampleI-offsetI)/4096)
		filteredI = sampleI - offsetI

		#-----------------------------------------------------------------------------
		# C) Root-mean-square method voltage
		#-----------------------------------------------------------------------------  
		sqV= filteredV * filteredV                #1) square voltage values
		sumV = sumV + sqV                               #2) sum

		#-----------------------------------------------------------------------------
		# D) Root-mean-square method current
		#-----------------------------------------------------------------------------   
		sqI = filteredI * filteredI                #1) square current values
		sumI = sumI + sqI                                #2) sum 

		#-----------------------------------------------------------------------------
		# E) Phase calibration
		#-----------------------------------------------------------------------------
		#phaseShiftedV = lastFilteredV + PHASECAL * (filteredV - lastFilteredV); 

		#-----------------------------------------------------------------------------
		# F) Instantaneous power calc
		#-----------------------------------------------------------------------------   
		instP = filteredV * filteredI          #Instantaneous Power
		sumP = sumP + instP                               #Sum  

		#-----------------------------------------------------------------------------
		# Stop after the next rising edge zero crossing
		# 
		#  
		#-----------------------------------------------------------------------------       
		rising_from_0_flag=0
		if ((sampleV < (ADC_COUNTS*0.6)) and (sampleV > (ADC_COUNTS*0.5)) and (sampleV>V1_data[-1024+start_point+sample_counter-1])): #Check for rising V and centered
			crossing_bool=1 #set flag, that it is inside crossing
			rising_from_0_flag=1
		else:
			crossing_bool=0
			
		if (rising_from_0_flag==1) and (crossing_bool==0):
			exit=1
		

		""""
		#-------------------------------------------------------------------------------------------------------------------------
		# 3) Post loop calculations
		#------------------------------------------------------------------------------------------------------------------------- 
		#Calculation of the root of the mean of the voltage and current squared (rms)
		#Calibration coefficients applied. 

		double V_RATIO = VCAL *((SupplyVoltage/1000.0) / (ADC_COUNTS));
		Vrms = V_RATIO * sqrt(sumV / numberOfSamples); 

		double I_RATIO = ICAL *((SupplyVoltage/1000.0) / (ADC_COUNTS));
		Irms = I_RATIO * sqrt(sumI / numberOfSamples); 

		#Calculation power values
		realPower = V_RATIO * I_RATIO * sumP / numberOfSamples;
		apparentPower = Vrms * Irms;
		powerFactor=realPower / apparentPower;

		#Reset accumulators
		sumV = 0;
		sumI = 0;
		sumP = 0;
		#--------------------------------------------------------------------------------------       
		"""
	Vrms = math.sqrt(sumV / sample_counter)*V1cal
	Irms = math.sqrt(sumI / sample_counter)*I1cal 
	RealP = V1cal*I1cal*sumP/sample_counter
	ApP=Vrms*Irms
	PF=RealP/ApP
	
	print ("Samples used: %s" % sample_counter)
	print ("V1_offset: %s" % offsetV)
	print ("I1_offset: %s" % offsetI)
	print ("V1_sample: %s" % sampleV)
	print ("I1_sample: %s" % sampleI)
	
	print ("V1_rms: %s" % Vrms)
	print ("I1_rms: %s" % Irms)
	print ("Watt: %s" % RealP)
	print ("VA: %s" % ApP)
	print ("P.F.: %s" % PF)
	#print ("P1: %s" % P)
	

while True:
	bytesToRead = ser.inWaiting()
	raw_data=ser.read(bytesToRead)
	if (bytesToRead>0):
		##print("-------------------------")
		##print "Bytes received: %s" % bytesToRead
		
		if (bytesToRead>=4):
			i=0
			sum=0
			while (i<4):
				sum=sum+ord(raw_data[i])
				i=i+1
			if (sum==1020):
				#os.system('clear')
				#print("Packet Header Detected - "), 
				packet_size=0
				packet_sum = ""

		packet_size=packet_size+bytesToRead
		#packet_sum.extend(raw_data)
		packet_sum=packet_sum+raw_data		
		
		#print "Packet_size: %s" % len(packet_sum)
		if (len(packet_sum)==3077):
			os.system('clear')
			print("Packet Header Detected - "), 
			print "Packet_size: %s -> Good Size" % len(packet_sum),
			i=0
			sum=0
			while (i<4):
				sum=sum+ord(packet_sum[i])
				i=i+1
			if (sum==1020):
				print(" - Packet conformed: "), 
				if (ord(packet_sum[4])==0x10):
					print("Samples Packet")
					packet_count=packet_count+1
					print ("Packet Count: %s" % packet_count),
					"""j=0
					while (j<bytesToRead):
						print (ord(packet_sum[j]))
						j=j+1
						"""
					
					V1 = []
					V2 = []
					V3 = []
					I1 = []
					I2 = []
					I3 = []
					j=0
					while (j<256):
						inte=unpack('h', packet_sum[5+(j*2):7+(j*2)])
						V1.append(inte[0])
						j=j+1
					while (j<512):
						inte=unpack('h', packet_sum[5+(j*2):7+(j*2)])
						V2.append(inte[0])
						j=j+1
					while (j<768):
						inte=unpack('h', packet_sum[5+(j*2):7+(j*2)])
						V3.append(inte[0])
						j=j+1
					while (j<1024):
						inte=unpack('h', packet_sum[5+(j*2):7+(j*2)])
						I1.append(inte[0])
						j=j+1
					while (j<1280):
						inte=unpack('h', packet_sum[5+(j*2):7+(j*2)])
						I2.append(inte[0])
						j=j+1
					while (j<1536):
						inte=unpack('h', packet_sum[5+(j*2):7+(j*2)])
						I3.append(inte[0])
						j=j+1
					
					V1_data=V1_data + V1
					V2_data=V2_data + V2
					V3_data=V3_data + V3
					I1_data=I1_data + I1
					I2_data=I2_data + I2
					I3_data=I3_data + I3
					
					if (packet_count>100):
						V1_data=V1_data[256:]
						V2_data=V2_data[256:]
						V3_data=V3_data[256:]
						I1_data=I1_data[256:]
						I2_data=I2_data[256:]
						I3_data=I3_data[256:]
					
					print ("Data lenght: %s" % len(V1_data))
					
					if (len(V1_data)>2048):
						Calc_VI()
					
					""""print "V1 data: ",
					print V1
					print "V2 data: ",
					print V2
					print "V3 data: ",
					print V3
					print "I1 data: ",
					print I1
					print "I2 data: ",
					print I2
					print "I3 data: ",
					print I3"""
					
					table = []
					
					array_of_data=V1
					signal = np.array(array_of_data[0:64], dtype=float) #Si se quiere menor separacion de canales de FFT, elegir menos muestras
					fourier = np.abs(np.fft.fft(signal))
					fourier_log = 20*np.log10(fourier)
					n = signal.size
					timestep = 0.0003125
					freq = np.fft.fftfreq(n, d=timestep)
					
					new_freq = ["FREQ"]+[int(floater) for floater in freq]
					new_fourier_log = ["V1  "]+[round(floater,1) for floater in fourier_log]
					table.append(new_fourier_log[0:(len(fourier)/4)+2])
					
					array_of_data=V2
					signal = np.array(array_of_data[0:64], dtype=float) #Si se quiere menor separacion de canales de FFT, elegir menos muestras
					fourier = np.abs(np.fft.fft(signal))
					fourier_log = 20*np.log10(fourier)
					n = signal.size
					timestep = 0.0003125
					freq = np.fft.fftfreq(n, d=timestep)
					new_fourier_log = ["V2  "]+[round(floater,1) for floater in fourier_log]
					table.append(new_fourier_log[0:(len(fourier)/4)+2])
					
					array_of_data=V3
					signal = np.array(array_of_data[0:64], dtype=float) #Si se quiere menor separacion de canales de FFT, elegir menos muestras
					fourier = np.abs(np.fft.fft(signal))
					fourier_log = 20*np.log10(fourier)
					n = signal.size
					timestep = 0.0003125
					freq = np.fft.fftfreq(n, d=timestep)
					new_fourier_log = ["V3  "]+[round(floater,1) for floater in fourier_log]
					table.append(new_fourier_log[0:(len(fourier)/4)+2])
					
					array_of_data=I1
					signal = np.array(array_of_data[0:64], dtype=float) #Si se quiere menor separacion de canales de FFT, elegir menos muestras
					fourier = np.abs(np.fft.fft(signal))
					fourier_log = 20*np.log10(fourier)
					n = signal.size
					timestep = 0.0003125
					freq = np.fft.fftfreq(n, d=timestep)
					new_fourier_log = ["I1  "]+[round(floater,1) for floater in fourier_log]
					table.append(new_fourier_log[0:(len(fourier)/4)+2])
					
					array_of_data=I2
					signal = np.array(array_of_data[0:64], dtype=float) #Si se quiere menor separacion de canales de FFT, elegir menos muestras
					fourier = np.abs(np.fft.fft(signal))
					fourier_log = 20*np.log10(fourier)
					n = signal.size
					timestep = 0.0003125
					freq = np.fft.fftfreq(n, d=timestep)
					new_fourier_log = ["I2  "]+[round(floater,1) for floater in fourier_log]
					table.append(new_fourier_log[0:(len(fourier)/4)+2])
					
					array_of_data=I3
					signal = np.array(array_of_data[0:64], dtype=float) #Si se quiere menor separacion de canales de FFT, elegir menos muestras
					fourier = np.abs(np.fft.fft(signal))
					fourier_log = 20*np.log10(fourier)
					n = signal.size
					timestep = 0.0003125
					freq = np.fft.fftfreq(n, d=timestep)
					new_fourier_log = ["I3  "]+[round(floater,1) for floater in fourier_log]
					table.append(new_fourier_log[0:(len(fourier)/4)+2])
					
					print ("---")
					print ("FFT")
					print ("---")
					
					print tabulate(table, new_freq[0:(len(fourier)/4)+2], floatfmt=".0f")
					
					#print table
					
					#j0=0
					#while (j0<len(fourier)/2): #Dado que la FFT es simetrica, con la mitad de los valores nos vale
					#	print "Freq: %s Value %s " % (freq[j0], int(fourier_log[j0]))
					#	j0=j0+1
					
					"""
					k0=0
					while (k0<6):
						print "loop %s" % k0
						signal = np.array(V1[0:64], dtype=float) #Si se quiere menor separacion de canales de FFT, elegir menos muestras
						fourier = np.abs(np.fft.fft(signal))
						fourier_log = 20*scipy.log10(fourier)
						n = signal.size
						timestep = 0.0003125
						freq = np.fft.fftfreq(n, d=timestep)
						j0=0
						while (j0<len(fourier)/2): #Dado que la FFT es simetrica, con la mitad de los valores nos vale
							#print "Freq: %s Value %s " % (freq[j0], fourier_log[j0])
							j0=j0+1
						k0=k0+1
				"""
			packet_size=0
			packet_sum = ""
		
		#else:
			#print "Packet_size: %s - Bad Size" % len(packet_sum)
			
				
			
