/* Example for synchonized measurements using both ADC present in Teensy 3.1
*  You can change the number of averages, bits of resolution and also the comparison value or range.
*/

#define ARM_MATH_CM4
#include <arm_math.h>
#include <ADC.h>
#include <IntervalTimer.h>

union u_tag {	
	int in;
	byte inby[2];
} u;

unsigned long time_packet;

//const int period0 = 78; // us 6400 Hz Interrupt
//const int period0 = 156; // us 6400 Hz Interrupt
const int period0 = 312; // us 3200 Hz 
//const int period0 = 624; // us 1600 Hz Interrupt
//const int period0 = 936; // us 1066 Hz Interrupt

const int FFT_SIZE=64;

const byte Pin_V1 = A9;
const byte Pin_I1 = A17;
const byte Pin_V2 = A8;
const byte Pin_I2 = A16;
const byte Pin_V3 = A7;
const byte Pin_I3 = A15;


int V1[256],V2[256],V3[256],I1[256],I2[256],I3[256];
int V1_alt[256],V2_alt[256],V3_alt[256],I1_alt[256],I2_alt[256],I3_alt[256];
int Sample_Counter=0;
int FFT_V1[64],FFT_V2[64],FFT_V3[64],FFT_I1[64],FFT_I2[64],FFT_I3[64];

byte Send_buff_byte[512];
byte Send_buff_time[4];
byte init_frame[4]={0xFF,0xFF,0xFF,0xFF};

bool cycle_alt=0;
byte ADC_Phase=3;

const int readPeriod = 1000; // us

ADC *adc = new ADC(); // adc object

IntervalTimer timer0;
int startTimerValue0 = 0;

byte readPin0,readPin1;

elapsedMicros time;

void setup() {

    pinMode(LED_BUILTIN, OUTPUT);
    pinMode(Pin_V1, INPUT);
    pinMode(Pin_I1, INPUT);

    Serial.begin(9600);
	
	digitalWrite(13,HIGH);
	
	delay(1000);
	
	Serial.println("Setup...");
    ///// ADC0 ////
    // reference can be ADC_REF_3V3, ADC_REF_1V2 (not for Teensy LC) or ADC_REF_EXT.
    //adc->setReference(ADC_REF_1V2, ADC_0); // change all 3.3 to 1.2 if you change the reference to 1V2

    adc->setAveraging(8); // set number of averages
    adc->setResolution(12); // set bits of resolution

    // it can be ADC_VERY_LOW_SPEED, ADC_LOW_SPEED, ADC_MED_SPEED, ADC_HIGH_SPEED_16BITS, ADC_HIGH_SPEED or ADC_VERY_HIGH_SPEED
    // see the documentation for more information
    adc->setConversionSpeed(ADC_HIGH_SPEED); // change the conversion speed
    // it can be ADC_VERY_LOW_SPEED, ADC_LOW_SPEED, ADC_MED_SPEED, ADC_HIGH_SPEED or ADC_VERY_HIGH_SPEED
    adc->setSamplingSpeed(ADC_HIGH_SPEED); // change the sampling speed

    // always call the compare functions after changing the resolution!
    //adc->enableCompare(1.0/3.3*adc->getMaxValue(ADC_0), 0, ADC_0); // measurement will be ready if value < 1.0V
    //adc->enableCompareRange(1.0*adc->getMaxValue(ADC_0)/3.3, 2.0*adc->getMaxValue(ADC_0)/3.3, 0, 1, ADC_0); // ready if value lies out of [1.0,2.0] V

    ////// ADC1 /////
    adc->setAveraging(8, ADC_1); // set number of averages
    adc->setResolution(12, ADC_1); // set bits of resolution
    adc->setConversionSpeed(ADC_HIGH_SPEED, ADC_1); // change the conversion speed
    adc->setSamplingSpeed(ADC_HIGH_SPEED, ADC_1); // change the sampling speed

    // always call the compare functions after changing the resolution!
    //adc->enableCompare(1.0/3.3*adc->getMaxValue(ADC_1), 0, ADC_1); // measurement will be ready if value < 1.0V
    //adc->enableCompareRange(1.0*adc->getMaxValue(ADC_1)/3.3, 2.0*adc->getMaxValue(ADC_1)/3.3, 0, 1, ADC_1); // ready if value lies out of [1.0,2.0] V

	adc->enableInterrupts(ADC_0);
	
	Serial.println("Starting Timers");

    // start the timers, if it's not possible, startTimerValuex will be false
    startTimerValue0 = timer0.begin(timer0_callback, period0);
    // wait enough time for the first timer conversion to finish (depends on resolution and averaging),
    // with 16 averages, 12 bits, and ADC_MED_SPEED in both sampling and conversion speeds it takes about 36 us.
    delayMicroseconds(25); // if we wait less than 36us the timer1 will interrupt the conversion
    // initiated by timer0. The adc_isr will restart the timer0's measurement.

    delay(100);
    Serial.println("end setup");
}

int value = 0;
int value2 = 0;

ADC::Sync_result result;

char c=0;

void Send_Serial()
{
	
	time_packet = micros();
	memcpy(Send_buff_time,&time_packet,4);
	Serial.write(init_frame, 4);
	Serial.write(0x10);
	//Serial.write(Send_buff_time, 4);
	
	
	if (cycle_alt==0)
	{
		int k=0;
		while (k<256)
		{
			u.in=V1_alt[k];
			Serial.write(u.inby[0]);
			Serial.write(u.inby[1]);
			k++;
		}
		
		k=0;
		while (k<256)
		{
			u.in=V2_alt[k];
			Serial.write(u.inby[0]);
			Serial.write(u.inby[1]);
			k++;
		}
		
		k=0;
		while (k<256)
		{
			u.in=V3_alt[k];
			Serial.write(u.inby[0]);
			Serial.write(u.inby[1]);
			k++;
		}
		
		k=0;
		while (k<256)
		{
			u.in=I1_alt[k];
			Serial.write(u.inby[0]);
			Serial.write(u.inby[1]);
			k++;
		}
		
		k=0;
		while (k<256)
		{
			u.in=I2_alt[k];
			Serial.write(u.inby[0]);
			Serial.write(u.inby[1]);
			k++;
		}
		
		k=0;
		while (k<256)
		{
			u.in=I3_alt[k];
			Serial.write(u.inby[0]);
			Serial.write(u.inby[1]);
			k++;
		}
	}
	
	
	if (cycle_alt==1)
	{
		int k=0;
		while (k<256)
		{
			u.in=V1[k];
			Serial.write(u.inby[0]);
			Serial.write(u.inby[1]);
			k++;
		}
		
		k=0;
		while (k<256)
		{
			u.in=V2[k];
			Serial.write(u.inby[0]);
			Serial.write(u.inby[1]);
			k++;
		}
		
		k=0;
		while (k<256)
		{
			u.in=V3[k];
			Serial.write(u.inby[0]);
			Serial.write(u.inby[1]);
			k++;
		}
		
		k=0;
		while (k<256)
		{
			u.in=I1[k];
			Serial.write(u.inby[0]);
			Serial.write(u.inby[1]);
			k++;
		}
		
		k=0;
		while (k<256)
		{
			u.in=I2[k];
			Serial.write(u.inby[0]);
			Serial.write(u.inby[1]);
			k++;
		}
		
		k=0;
		while (k<256)
		{
			u.in=I3[k];
			Serial.write(u.inby[0]);
			Serial.write(u.inby[1]);
			k++;
		}
	}
	
}

void loop() {
	
	if (Sample_Counter>255)
	{
		//Serial.println("Cycle Done");
		digitalWrite(13,cycle_alt);
		Sample_Counter=0;
		cycle_alt=!cycle_alt;
		//Make_FFT;
		if (Serial.dtr()) //Sends data if the Serial port is opened on the COmputer / RPi
			Send_Serial();
	}
}

// This function will be called with the desired frequency
// start the measurement
// in my low-res oscilloscope this function seems to take 1.5-2 us.
void timer0_callback(void) {


	if (ADC_Phase==1)
	{
		readPin0=Pin_V1;
		readPin1=Pin_I1;
	}
	else if (ADC_Phase==2)
	{
		readPin0=Pin_V2;
		readPin1=Pin_I2;
	}
	else if (ADC_Phase==3)
	{
		readPin0=Pin_V3;
		readPin1=Pin_I3;
	}

	
	adc->startSynchronizedSingleRead(readPin0, readPin1); // also: startSingleDifferential, analogSynchronizedRead, analogSynchronizedReadDifferential


}

// when the measurement finishes, this will be called
// first: see which pin finished and then save the measurement into the correct buffer
void adc0_isr() {

    result = adc->readSynchronizedSingle();
    // if using 16 bits and single-ended is necessary to typecast to unsigned,
    // otherwise values larger than 3.3/2 will be interpreted as negative
	
	int result1,result2;
	result1=result.result_adc0;
	result2=result.result_adc1;
	
	
	if (cycle_alt==0)
	{
		if (ADC_Phase==1)
		{
			V1[Sample_Counter]=result.result_adc0;
			I1[Sample_Counter]=result.result_adc1;
		}
		else if (ADC_Phase==2)
		{
			V2[Sample_Counter]=result.result_adc0;
			I2[Sample_Counter]=result.result_adc1;
		}
		else if (ADC_Phase==3)
		{
			V3[Sample_Counter]=result.result_adc0;
			I3[Sample_Counter]=result.result_adc1;
		}
	}
	
	
	
	else
	{
		if (ADC_Phase==1)
		{
			V1_alt[Sample_Counter]=result.result_adc0;
			I1_alt[Sample_Counter]=result.result_adc1;
		}
		else if (ADC_Phase==2)
		{
			V2_alt[Sample_Counter]=result.result_adc0;
			I2_alt[Sample_Counter]=result.result_adc1;
		}
		else if (ADC_Phase==3)
		{
			V3_alt[Sample_Counter]=result.result_adc0;
			I3_alt[Sample_Counter]=result.result_adc1;
		}
	}
	
	
	ADC_Phase++;
	if (ADC_Phase>3)
	{
		
		ADC_Phase=1;
		//Serial.print("Sample: ");
		//Serial.println(Sample_Counter);
		Sample_Counter++;
		
	}
	
	else
	{
		if (ADC_Phase==2)
		{
			readPin0=Pin_V2;
			readPin1=Pin_I2;
		}
		else if (ADC_Phase==3)
		{
			readPin0=Pin_V3;
			readPin1=Pin_I3;
		}

		
		adc->startSynchronizedSingleRead(readPin0, readPin1); // also: startSingleDifferential, analogSynchronizedRead, analogSynchronizedReadDifferential
	}
	
	/*
	Serial.print("Cycle: ");
	Serial.println(cycle_alt);
	
	Serial.print("Phase: ");
	Serial.println(ADC_Phase);
	
	
	
			
	Serial.print(result.result_adc0);
	Serial.print(" ");
	Serial.println(result.result_adc0);
	*/

}